function [y_traj,u_traj] = SimulatePendulum(F_lqr,perfectStateMeasurement)
% Simulation of simple pendulum example (Sec. 4).
%
% Input:
%   F_lqr                   LQR controller gain
%   perfectStateMeasurement Perfect state measurements? true/false
%
% Output:
%   y_traj      output data {y_k}
%   u_traj      input data {u_k}
%
% --
% Self-tuning LQR, Trimpe et al. IFAC World Congress 2014 paper
%
% Copyright:
% Sebastian Trimpe
% Max-Planck-Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [13.11.13]   first version to be published


%% Configuration

% Simulation parameters
timeHorz = 300;         % time horizon for simulation, []=s

% Pendulum model
[m_true] = ConfigPendulumModel;     % true model
[Ad,Bd,Cd,Dd,Ts] = ssdata(m_true.sysd);
n_state = length(Ad);
n_input = size(Bd,2);



%% Closed-loop system
% Plant model under state feedback with controller F_lqr.
% input: (w_k, v_k), process and measurement noise (v_k is 0 for the case
%                      of perfect state measurements)
% output: (y_k, u_k), state and input trajectory for evaluating cost
A_cl = (Ad+Bd*F_lqr);
B_cl = [eye(n_state), Bd*F_lqr];
C_cl = [eye(n_state); F_lqr];
D_cl = [zeros(n_state), eye(n_state); zeros(n_input,n_state), F_lqr];
sys_cl = ss(A_cl, B_cl, C_cl, D_cl, Ts);


%% Simulation
% Time trajectory
t_traj = 0:Ts:timeHorz;
K = length(t_traj);

% Input signals: noise.
% Process noise variance: sigma_n^2*Bd*Bd'
input_w = randn(K,1)*Bd'*sqrt(m_true.sigma_n_sq);     % process noise

if perfectStateMeasurement
    input_v = zeros(K,2);
else
    input_v = randn(K,2)*sqrt(m_true.V_n);     % measurement noise
    % Can use sqrt() since V_n is diagonal.
end;

% Initial state
x0 = [0; 0];

% Check if system is stable.
if any(abs(eig(sys_cl))>1)
    disp('The closed-loop system is unstable.');
    y_traj = inf;
    u_traj = inf;
    return;
end;

% Simulation
[y_sim,t_sim] = lsim(sys_cl,[input_w, input_v],t_traj,x0);

y_traj = y_sim(:,1:n_state);
u_traj = y_sim(:,n_state+1);
